/*********   ECE 4760 FINAL PROJECT ****************
*				Pronation Detection				   *
*					5/5/2011                       *
*             Wen Jie Zhou wz54                    *
*             Yilok Wong   ylw3                    *
***************************************************/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <avr/eeprom.h> 
#include <avr/pgmspace.h>
#include "lcd_lib.h"
#include <stdlib.h>
#include <string.h>
#include "Final_Project.h"
#include "calculations.h"

//fatfs libraries
#include "fatfs/ff.h"
#include "fatfs/ff.c"
#include "fatfs/mmc.c"
#include "fatfs/diskio.h"
#include "sdcard.c"

// serial communication library
#include "uart.h"
//UART file descriptor
//putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//timeout values for each task
#define t1 50	//sensor timer
#define t2 100	//sd timer
#define diskInterval 10  		// Interval for disk_timerproc (NEEDS to be 10ms)
   
#define begin {
#define end }

#define sdEnable 0
#define mainDebug 1

//ADC variables for 10 bit accuracy 
char AinH, AinL; //high and low registers for ADC
unsigned int reading; //accelerometer reading
int addr;

//Internal ADC functions
unsigned int accel_data1; 		//raw A to D number for first accelerometer
unsigned int accel_data2; 		//raw A to D number for second accelerometer
unsigned int force_data1;		//raw A to D number for first FSR
unsigned int force_data2;		//raw A to D number for second FSR
unsigned int force_data3; 		//raw A to D number for third FSR
unsigned int gyro_data1;		//raw A to D number for first gyro
unsigned int gyro_data2;		//raw A to D number for second gyro

//Timers
volatile unsigned int time1;	//sensor timeout counter 
volatile unsigned int time2;	//sd timeout couter
volatile char diskTime;      //disk_timerproc timer
unsigned int ticks ;			// running time

//Fat Flags
unsigned char diskInitFlag;
unsigned char diskMountFlag;

// FatFS variables
FIL logFile;
FIL* logPtr;
FATFS fs;
FATFS* fsPtr;

//**********************************************************
/* Update the current system in day/hours/minutes at a 1ms */
void incrementSysTime(void) {
	if (++millis == 1000) {
		millis = 0;
		secs++;
	}
	if (secs == 60) {
		secs = 0;
		mins++;
	}
	if (mins == 60) {
		mins = 0;
		hours++;
	}
}

//**********************************************************
//timer 0 compare ISR
ISR (TIMER0_COMPA_vect){
	incrementSysTime();
	
	//Decrement the  time if they are not already zero
	if (time1>0)	--time1;
	if (time2>0)	--time2;
	if (stepTimeout>0)	--stepTimeout;
	if (gyroTime>0) --gyroTime;
	/* ------- Disk Timer ---------- */
	if (diskTime>0) --diskTime;
	if (diskTime == 0) {
		diskTime = diskInterval;
		//needs to run async, call from interrupt
		#if sdEnable
		disk_timerproc();
		#endif
	}
}  


//**********************************************************          
//Read ADC
unsigned int read_ADC(void){
	// start a conversion
	ADCSRA |= (1<<ADSC);

	while (ADCSRA & (1<<ADSC)) { //do nothing 
	}
	
	AinL = ADCL;
	AinH = ADCH;
	
	reading = AinL;
	reading = (AinH << 8) | reading;

	return reading;
}

//**********************************************************          
//Read accelerometers
unsigned int read_A1(void){
	// First accelerometer on A0
	ADMUX =(0x00) |(1<<REFS0);
	accel_data1 = read_ADC();
	return accel_data1;
}
unsigned int read_A2(void){
	// Second accelerometer on A1
	ADMUX = (1<<MUX0) |(1<<REFS0);
	accel_data2 = read_ADC();
	return accel_data2;
}

void read_accel(void){
	
	accel_data1 = read_A1();
	accel_data2 = read_A2();
	
	// Store info into temporary arrays
	accel1Array[arrayNumber] = accel_data1;
	accel2Array[arrayNumber] = accel_data2;
	
	#if mainDebug
	printf("A_1: %u A_2: %u ", accel_data1,accel_data2);
	#endif
}



//**********************************************************          
//Read force sensors
unsigned int read_F1(void){
	// Force sensor on A2
	ADMUX = (1<<MUX1) |(1<<REFS0);
	force_data1 = read_ADC();
	return force_data1;
}
unsigned int read_F2(void){
	// Force sensor on A3
	ADMUX = (1<<MUX1) | (1<<MUX0) |(1<<REFS0);
	force_data2 = read_ADC();
	return force_data2;
}
unsigned int read_F3(void){
	// Force sensor on A4
	ADMUX = (1<<MUX2) |(1<<REFS0);
	force_data3 = read_ADC();
	return force_data3;
}

void read_force(void){
	force_data1 = read_F1();
	force_data2 = read_F2();
	force_data3 = read_F3();
	
	// Store info into temporary arrays
	force1Array[arrayNumber] = force_data1;
	force2Array[arrayNumber] = force_data2;
	force3Array[arrayNumber] = force_data3;
	
	#if mainDebug
	printf("F_Heel: %u F_Ball: %u F_Outs: %u ",force_data1,force_data2,force_data3);
	#endif

}


//**********************************************************          
//Read gyro
unsigned int read_G1(void){
	// Gyro on A5	
	ADMUX = (1<<MUX2) | (1<<MUX0) | (1<<REFS0);
	gyro_data1 = read_ADC();
	gyro1Array[arrayNumber] = gyro_data1;
	return gyro_data1;
}
//Read gyro
unsigned int read_G2(void){
	// Gyro on A6	
	ADMUX = (1<<MUX2) | (1<<MUX1) | (1<<REFS0);
	gyro_data2 = read_ADC();
	gyro2Array[arrayNumber] = gyro_data2;
	return gyro_data2;
}
void read_gyro(void){

	gyro_data1 = read_G1();
	gyro_data2 = read_G2();
	
	
	gyro1Array[arrayNumber] = gyro_data1;
	gyro2Array[arrayNumber] = gyro_data2;
	
	#if mainDebug
	printf("G_X: %u G_Y: %u\n\r",gyro_data1,gyro_data2);
	#endif
}
//**********************************************************          
//Initialize SPI
void SPIInit(void){
	//set up i/o data direction
	//DDRB.4 = 1; //output chip select for DAC (notSYNC)
	//DDRB.5 = 1; //output MOSI to ADC
	//DDRB.6 = 0; //input MISO from ADC
	//DDRB.7 = 1; //output SCLK   
	DDRB = (1<<DDB4) | (1<<DDB5) | (1<<DDB7) ; 
	
	// initialize notSYNC
	SET(PORTB,4) ;


	// set up the SPI module: SPI enabled, MSB first, master mode,
	//  clock polarity and phase = 0, F_osc/16
	SPCR = ( 1 << SPE ) | ( 1 << MSTR ) | ( 1 << SPR1 );
	//SPSR = (1<<SPI2X) ; //SPSR = 1; 
}



//********************************************************** 
//Set it all up
void initialize(void){

	//On LED
	//set up the LED port
	DDRD = (1<<PORTD2);	// PORT D.2 is an ouput
	PORTD = (0<<PORTD2); // Turn on LED
	//ADC
	//init the A to D converter 
	//channel zero/ left adj /EXTERNAL Aref
	//!!!CONNECT Aref jumper!!!!
	DDRA = 0x00; 			//protect output pull up r
	ADMUX = (1<<ADLAR)|(1<<REFS0);      
	//Default pin0 for Accelerometer A/D	
	//enable ADC and set prescaler to 1/128*16MHz=125,000
	//and clear interupt enable
	ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); 

	//Internal Timers       
	//set up timer 0 for 1 mSec timebase 
	TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
	OCR0A = 249;  		//set the compare register to 250 time ticks
	//set prescalar to divide by 64 
	TCCR0B= 3; 	
	// turn on clear-on-match
	TCCR0A= (1<<WGM01) ;

	//Software Timers
	//init the task timer
	time1=t1;  

	//init time
	millis = 0;
	secs = 0;
	mins = 0;
	hours = 0;
	day = 0; //dummy day
	month = 0; //dummy month
	year = 1; //dummy year 

	// Init SD card variables
	openFlag = 0;
	diskInitFlag = 0;
	diskMountFlag = 0;
	counter = 0;
	fileCounter = 0;
	diskTime = diskInterval;
	

	//Other Initializations
	initCalc();	//calculation variables init
	uart_init();
	stdout = stdin = stderr = &uart_str;

	fprintf(stdout,"\n\rStarting...\n\r");

	//Enable Interrupts
	sei();
} 

//**********************************************************       
//Entry point and task scheduler loop
int main(void){ 
	initialize();
	#if sdEnable
	init_fs(); //init disk and mount filesystem
	openLog(day,hours);
	#endif
	//testSDCard();
	//main task scheduler loop 
	while(1){
		if (time1==0){	
			#if mainDebug
			read_accel();	
			read_force();
			read_gyro();
			#endif
			#if !mainDebug
			stepDetect();
			//pronationDetection();
			#endif
			arrayNumber++;

		
			/* If array buffer full, write to SD card */
			if(arrayNumber == dataWriteInterval){
				#if sdEnable
				//openLog(day,hours);
				writeSDcard();
				#endif
				arrayNumber = 0;
			}
			time1=t1; // Reset timer here so ISR doesn't cause dumb things to happen
		}
		if (gyroTime==0){
			//measureTilt();
		}
	}
}    
