/* Calculation utilities */

#include "Final_Project.h"
#include "calculations.h"
#include "sdcard.h"
#include <stdio.h>

#define AriseThresh 980	//raise for more forceful triggering
#define AfallThresh	300	//lower for more forceful triggering
#define FThresh 80

#define TRACKING	1
#define STEPUP		2
#define STEPDOWN	3
#define STEPTAKEN	4

#define HEEL			1
#define CHKPRONATION	2
#define PRONATION 		3

#define tOutStep 800
#define gTime 100

unsigned int A1, A2, F1, F2, F3, G1, G2;
unsigned int pA1, pA2, pF1, pF2, pF3, pG1, pG2;
int Xangle,Yangle;
int diffA,diffB;

char state, proState;

void pullValues(void){
	//Reads cannot be individually called or else they'll
	//ruin the sync of the SD Card printout array
	A1 = read_A1();
	A2 = read_A2();
	F1 = read_F1();
	F2 = read_F2();
	F3 = read_F3();
	G1 = read_G1();
	G2 = read_G2();
}

void stepDetect(void){
	pullValues();
	switch(state){
		case TRACKING:
			//printf("Tracking...\n\r");
			//printf("A1: %u AThresh: %u\n\r",A1,AriseThresh);
			//Possible step taken
			if ((A1>AriseThresh)||(A2>AriseThresh)){
				state = STEPUP;
			}
			break;
		case STEPUP:
			printf("At STEPUP! \n\r");
			//Second sample should also be high (for possible noise)
			pullValues();
			while((A1>AriseThresh)&&(A2>AriseThresh)){
				//wait for oscillations to die out
				if(stepTimeout == 0){
					printf("Timeout\n\r");
					//no downstep measured, this is not considered a step
					state = TRACKING;
					break;
				}
			}
			stepTimeout = tOutStep;
			if (state != TRACKING) state = STEPTAKEN;
			break;
		case STEPTAKEN:
			stepsTaken++;
			printf("Steps taken: %u\n\r",stepsTaken);
			state = TRACKING;
			break;
		default:
			//Something went wrong, restart
			state = TRACKING;
			break;
	}

}


//called every 10ms
//0.625 deg/sec per 1 ADC int
void measureTilt(){
	pullValues();

	Xangle += ((G1*4)>>3) - 159;  //same as diffA*0.625
	Yangle += ((G2*4)>>3) - 159;  //same as diffA*0.625
	printf("X angle: %d Y angle: %d\n\r",Xangle,Yangle);
}

void pronationDetection(void){
	pullValues();
	switch(proState){
		case HEEL:
			if (F1 > 250){
				proState = CHKPRONATION;
			}
			break;
		case CHKPRONATION:
			stepTimeout = tOutStep;
			pullValues();
			while((F2<80)&&(F3<250)){
				//wait for oscillations to die out
				pullValues();
				if(stepTimeout == 0){
					printf("Timeout\n\r");
					//no downstep measured, this is not considered a step
					state = HEEL;
					break;
				}
			}
			if (proState != HEEL) proState = PRONATION;
			break;
		case PRONATION:
			if(F2>80) printf("You are pronated\n\r");
			else printf("You are supinated\n\r");
			proState = HEEL;
			break;
		default:
			//Something went wrong, restart
			proState = HEEL;
			break;
	}
}

int getWeight(){
	
	return 0;
}

void initCalc(void){
	A1 = 0;	A2 = 0;
	F1 = 0;	F2 = 0;	F3 = 0;
	G1 = 0; G2 = 0;
	pA1 = 0; pA2 = 0;
	pF1 = 0; pF2 = 0; pF3 = 0;
	pG1 = 0; pG2 = 0;
	state = TRACKING;
	proState = HEEL;
	stepTimeout = tOutStep;
	stepsTaken = 0;
	gyroTime = gTime;
	Xangle = 0;
	Yangle = 0;
}
